#include <Agents/Agent.hpp>
#include <Environment/PictureEnvironment.hpp>

RepaintAgent::RepaintAgent(PictureEnvironment* iEnvironment, int iDotSize, int iMoveSize, int iTolerance) :
	Agent(iEnvironment, iDotSize, iMoveSize)
{
	init();
	fTolerance = iTolerance;
	//Initalize list of possible movement vectors
	fMovements = std::vector<std::vector<int>>();
	fValidMovementIndexes = std::vector<int>();
	fCurrentMovement = std::vector<int>();
	for (int i = -fMoveSize; i <= fMoveSize; i++)
	{
		for (int j = -fMoveSize; j <= fMoveSize; j++)
		{
			fMovements.push_back({ i, j });
		}
	}
}

void RepaintAgent::init()
{
	int vWidth = fEnvironment->getWidth();
	int vHeight = fEnvironment->getHeight();

	fXPos = (((double)rand() / RAND_MAX) * vWidth);
	fYPos = (((double)rand() / RAND_MAX) * vHeight);
	fColor = sf::Color(fEnvironment->getCellOriginal(fXPos, fYPos));
}

void RepaintAgent::move()
{
	int vXMove;
	int vYMove;
	int fMovementsCount = fMovements.size();
	//Only move on cells having a sufficiently close color
	fValidMovementIndexes.clear();
	for (int i = 0; i < static_cast<int>(fMovementsCount); i++)
	{
		vXMove = fXPos + fMovements.at(i).at(0);
		vYMove = fYPos + fMovements.at(i).at(1);
		if (
			vXMove > 0 && vYMove > 0 && vXMove < fEnvWidth && vYMove < fEnvHeight && fEnvironment->getCell(vXMove, vYMove) != fColor.toInteger() && colorCloseEnough(fEnvironment->getCellOriginal(vXMove, vYMove)))
		{
			fValidMovementIndexes.push_back(i);
		}
	}
	if (fValidMovementIndexes.size() > 0)
	{
		int vRandIndex = rand() % static_cast<int>(fValidMovementIndexes.size());
		fCurrentMovement = fMovements.at(fValidMovementIndexes.at(vRandIndex));
		fXPos += fCurrentMovement.at(0);
		fYPos += fCurrentMovement.at(1);
		// std::cout << "moved " << fXPos << ":" << fYPos;
	}
	else
	{
		init();
	}
}

bool RepaintAgent::colorCloseEnough(sf::Uint32 iColor)
{
	sf::Uint32 va = iColor;
	sf::Uint32 vb = (int)fColor.toInteger();
	return abs((int)(va >> 24) - (int)(vb >> 24)) <= fTolerance &&		  //R
		abs((int)(va << 8 >> 24) - (int)(vb << 8 >> 24)) <= fTolerance && //G
		abs((int)(va << 16 >> 24) - (int)(vb << 16 >> 24)) <= fTolerance; //B
}

void RepaintAgent::paint()
{
	int vHalfBrush = fDotSize / 2;
	int vXLoopIndex = fXPos - vHalfBrush;
	int vYLoopIndex = fYPos - vHalfBrush;
	int vXMaxIndex = fXPos + vHalfBrush;
	int vYMaxIndex = fYPos + vHalfBrush;
	while (vXLoopIndex < (vXMaxIndex))
	{
		while (vYLoopIndex < (vYMaxIndex))
		{
			fEnvironment->paint(vXLoopIndex, vYLoopIndex, fColor);
			// fEnvironment->paint(vXLoopIndex, vYLoopIndex, fColor);
			vYLoopIndex++;
		}
		vYLoopIndex = fYPos - vHalfBrush;
		vXLoopIndex++;
	}
}

void RepaintAgent::loop()
{
	move();
	this->paint();
}
